/*
 *  c4brd.cpp
 *
 *  Determines what connect four move to play.
 *  There are faster algorithms, but this one is less code
 *
 *  Ken Reneris     12/06/2002
 *
 */

#include "std.h"

//
// Global data
//


#define LineS4(d,e,f,g)       _l4[Mul27[d] + Mul9[e] + Mul3[f] + g]
#define LineS5(c,d,e,f,g)     _l5[Mul81[c] + Mul27[d] + Mul9[e] + Mul3[f] + g]
#define LineS6(b,c,d,e,f,g)   _l6[Mul243[b] + Mul81[c] + Mul27[d] + Mul9[e] + Mul3[f] + g]
#define LineS7(a,b,c,d,e,f,g) _l7[Mul729[a] + Mul243[b] + Mul81[c] + Mul27[d] + Mul9[e] + Mul3[f] + g]


const INTN MoveOrder[] = { 3, 2, 4, 1, 5, 0, 6, -1 };


const INT8 _l7[2187] = {
    #include"_l7.cpp"
};

const INT8 _l6[729] = {
    #include"_l6.cpp"
};


const INT8 _l5[243] = {
    #include"_l5.cpp"
};


const INT8 _l4[81] = {
    #include"_l4.cpp"
};


#define WINNER_VALUE    (127-16*4)
#define MAXMOVES        (R_BOARD_SIZE*R_BOARD_SIZE)

// Tables to multiply cell value by powers of 3
const UINTN Mul3[]    = {    0*3,    1*3,   2*3  };
const UINTN Mul9[]    = {    0*9,    1*9,   2*9  };
const UINTN Mul27[]   = {   0*27,   1*27,   2*27 };
const UINTN Mul81[]   = {   0*81,   1*81,   2*81 };
const UINTN Mul243[]  = {  0*243,  1*243,  2*243 };
const UINTN Mul729[]  = {  0*729,  1*729,  2*729 };



/////////////////////////////////////////////////////////////////////////////
//
// C4Brd
//

const C4Brd& C4Brd::operator=(C4Brd &s1)
{
    m_Color = s1.m_Color;
    m_MoveNo = s1.m_MoveNo;
    memcpy (m_Col, &s1.m_Col, sizeof(m_Col));
    memcpy (m_Brd, &s1.m_Brd, sizeof(m_Brd));
    return *this;
}


VOID C4Brd::InitialBoard()
// Construct starting game board
{
    UINTN       i;

    memset(m_Brd, R_EMPTY, sizeof(m_Brd));

    for (i=0; i < R_BOARD_SIZE; i++) {
        m_Col[i] = BPos(i,R_BOARD_SIZE-1);
    }

    m_Color = R_BLACK;
    m_MoveNo = 0;
    m_MaxDepth = 0;
}


INTN C4Brd::Get(UINTN x, UINTN y)
{
    return (x < R_BOARD_SIZE && y < R_BOARD_SIZE) ? m_Brd[BPos(x,y)] : R_MAX_TYPE;
}

VOID C4Brd::NextPlayer()
{
    m_Color ^= R_FLIP;
}


BOOL C4Brd::CanMove(UINTN Col)
{
    return (Col < R_BOARD_SIZE && m_Col[Col] >= 0);
}

INTN C4Brd::ColY(UINTN Col)
{
    return m_Col[Col] / R_BOARD_SIZE;
}




/////////////////////////////////////////////////////////////////////////////
//
// Evaluate board and locate good moves (non-ending)
//

INTN C4Brd::RootMoveSearch(UINTN Depth)
{
    INTN        BestScore, BestMove, BestCount, Index;
    INTN        Col, Score;

    m_MaxDepth = Depth;

    // If first 2 moves, just random
    if (m_MoveNo < 2) {
        return rand() % R_BOARD_SIZE;
    }

    // Find best evaled move
    BestMove = -1;
    BestScore = MIN_INTN;
    BestCount = 1;

    //
    for(Index=0; ((Col=MoveOrder[Index]) != -1); Index++) {

        if (m_Col[Col] >= 0) {
            CApp->DrawProgressBar(Index+1);

            DropChip(Col);
            Score = -AlphaBeta(Depth, MIN_INTN, MAX_INTN);
            PopChip(Col);

            if (Score > BestScore) {
                BestMove = Col;
                BestScore = Score;
                BestCount = 1;

            } else if (Score == BestScore) {
                // If equal, make a random choice
                BestCount += 1;
                if ((rand() & 0xffff) < 0x10000/BestCount) {
                    BestMove = Col;
                }
            }
        }

    }
    return BestMove;
}

INTN C4Brd::AlphaBeta(UINTN Depth, INTN Alpha, INTN Beta)
// AlphaBeta search
{
    INTN        *Move, Col, Score, SScore, SCount;

    //
    // Evaluate last move
    //

    Score = Eval();

    // If last move wins the game, return it
    if (Score > WINNER_VALUE) {
        return - ((MAX_INTN - (m_MoveNo << 8)) >> 4);
    }

    // End of depth?
    if (Depth == 0 || m_MoveNo == MAXMOVES) {
        return -Score;
    }

    //
    // Flip player and check moves
    //

    m_Color ^= R_FLIP;
    SScore = Alpha;
    SCount = 0;

    Depth -= 1;
    for(Move=(INTN *) MoveOrder; ((Col=*Move) != -1); Move++) {
        if (m_Col[Col] >= 0) {

            DropChip(Col);
            Score = -AlphaBeta(Depth, -Beta, -Alpha);
            PopChip(Col);       // Undo move

            // If better for us, then remember it
            if (Score > Alpha) {
                Alpha = Score;

            // If tied, then bias the score to show it
            } else if (Score == Alpha) {

                if (Score != SScore) {
                    SScore = Score;
                    SCount = 0;
                }
                SCount += 1;
            }

            if (Alpha >= Beta) {
                break;
            }
        }

    }


    m_Color ^= R_FLIP;
    return Alpha + SCount;
}

BOOL C4Brd::GameOver()
{
    return (m_MoveNo == MAXMOVES || Winner() != R_EMPTY);
}


UINTN C4Brd::Winner()
// Check for a winner
{
    UINTN       Color;
    INTN        Score;

    Score = Eval();
    Color = R_EMPTY;
    if (Score > WINNER_VALUE)           Color = m_Color;
    else if (Score < -WINNER_VALUE)     Color = m_Color ^ R_FLIP;
    return Color;
}

INTN C4Brd::Eval()
{
    INTN        Score;

    Score = 0;

    // Horz lines
    Score += LineS7(Chip(0,0), Chip(1,0), Chip(2,0), Chip(3,0), Chip(4,0), Chip(5,0), Chip(6,0));
    Score += LineS7(Chip(0,1), Chip(1,1), Chip(2,1), Chip(3,1), Chip(4,1), Chip(5,1), Chip(6,1));
    Score += LineS7(Chip(0,2), Chip(1,2), Chip(2,2), Chip(3,2), Chip(4,2), Chip(5,2), Chip(6,2));
    Score += LineS7(Chip(0,3), Chip(1,3), Chip(2,3), Chip(3,3), Chip(4,3), Chip(5,3), Chip(6,3));
    Score += LineS7(Chip(0,4), Chip(1,4), Chip(2,4), Chip(3,4), Chip(4,4), Chip(5,4), Chip(6,4));
    Score += LineS7(Chip(0,5), Chip(1,5), Chip(2,5), Chip(3,5), Chip(4,5), Chip(5,5), Chip(6,5));
    Score += LineS7(Chip(0,6), Chip(1,6), Chip(2,6), Chip(3,6), Chip(4,6), Chip(5,6), Chip(6,6));

    // Vert lines
    Score += LineS7(Chip(0,0), Chip(0,1), Chip(0,2), Chip(0,3), Chip(0,4), Chip(0,5), Chip(0,6));
    Score += LineS7(Chip(1,0), Chip(1,1), Chip(1,2), Chip(1,3), Chip(1,4), Chip(1,5), Chip(1,6));
    Score += LineS7(Chip(2,0), Chip(2,1), Chip(2,2), Chip(2,3), Chip(2,4), Chip(2,5), Chip(2,6));
    Score += LineS7(Chip(3,0), Chip(3,1), Chip(3,2), Chip(3,3), Chip(3,4), Chip(3,5), Chip(3,6));
    Score += LineS7(Chip(4,0), Chip(4,1), Chip(4,2), Chip(4,3), Chip(4,4), Chip(4,5), Chip(4,6));
    Score += LineS7(Chip(5,0), Chip(5,1), Chip(5,2), Chip(5,3), Chip(5,4), Chip(5,5), Chip(5,6));
    Score += LineS7(Chip(6,0), Chip(6,1), Chip(6,2), Chip(6,3), Chip(6,4), Chip(6,5), Chip(6,6));


    // Diag lower-left to upper-right
    Score += LineS4(Chip(0,3), Chip(1,2), Chip(2,1), Chip(3,0));
    Score += LineS5(Chip(0,4), Chip(1,3), Chip(2,2), Chip(3,1), Chip(4,0));
    Score += LineS6(Chip(0,5), Chip(1,4), Chip(2,3), Chip(3,2), Chip(4,1), Chip(5,0));
    Score += LineS7(Chip(0,6), Chip(1,5), Chip(2,4), Chip(3,3), Chip(4,2), Chip(5,1), Chip(6,0));
    Score += LineS6(Chip(1,6), Chip(2,5), Chip(3,4), Chip(4,3), Chip(5,2), Chip(6,1));
    Score += LineS5(Chip(2,6), Chip(3,5), Chip(4,4), Chip(5,3), Chip(6,2));
    Score += LineS4(Chip(3,6), Chip(4,5), Chip(5,4), Chip(6,3));

    // Daig lower-right to upper left
    Score += LineS4(Chip(3,0), Chip(4,1), Chip(5,2), Chip(6,3));
    Score += LineS5(Chip(2,0), Chip(3,1), Chip(4,2), Chip(5,3), Chip(6,4));
    Score += LineS6(Chip(1,0), Chip(2,1), Chip(3,2), Chip(4,3), Chip(5,4), Chip(6,5));
    Score += LineS7(Chip(0,0), Chip(1,1), Chip(2,2), Chip(3,3), Chip(4,4), Chip(5,5), Chip(6,6));
    Score += LineS6(Chip(0,1), Chip(1,2), Chip(2,3), Chip(3,4), Chip(4,5), Chip(5,6));
    Score += LineS5(Chip(0,2), Chip(1,3), Chip(2,4), Chip(3,5), Chip(4,6));
    Score += LineS4(Chip(0,3), Chip(1,4), Chip(2,5), Chip(3,6));

    // Tables are for white player
    if (m_Color == R_BLACK) {
        Score = -Score;
    }

    return Score;
}
