/*
 *  c4brd.h
 *
 *  Implements a connect four game board.
 *
 *  Ken Reneris     12/06/2002
 *
 */


class C4Brd;

//
//
//


#define R_BLACK         0       // A black stone
#define R_EMPTY         1       // A empty space
#define R_WHITE         2       // A white stone
#define R_MAX_TYPE      3

#define R_FLIP          2       // XOR this bit to flip between white & black


#define R_BOARD_SIZE    7       // Connect four is a 7x7 board

#define BPos(x,y)       ((y)*R_BOARD_SIZE + (x))
#define Chip(x,y)       m_Brd[BPos(x,y)]


class C4Brd {
public:
    const C4Brd& operator=(C4Brd &s1);                  // Copy
    VOID InitialBoard();                                // Setup initial board

    BOOL CanMove(UINTN Col);
    INTN ColY(UINTN Col);

    INTN Get(UINTN x, UINTN y);                         // Return stone type is cell
    VOID NextPlayer();                                  // Switch to next player

    BOOL GameOver();
    UINTN Winner();

    INTN RootMoveSearch(UINTN Depth);                  // Find a move
    INTN Eval();

    inline VOID DropChip(INTN Col)
    {
        m_Brd[m_Col[Col]] = m_Color;
        m_Col[Col] -= R_BOARD_SIZE;
        m_MoveNo += 1;
    }

    inline VOID PopChip(INTN Col)
    {
        m_Col[Col] += R_BOARD_SIZE;
        m_Brd[m_Col[Col]] = R_EMPTY;
        m_MoveNo -= 1;
    }


private:
    // Eval dispatch
    VOID GetMoveOrder(INTN *Order);
    INTN AlphaBeta(UINTN Depth, INTN Alpha, INTN Beta); // AlphaBeta search

public:
    INTN            m_Color;                            // Current turn color
    INTN            m_MoveNo;                           // Count of moves
    INTN            m_MaxDepth;                         // Max depth searching too
    INTN            m_Col[R_BOARD_SIZE];                // Position on each column
    CHAR            m_Brd[R_BOARD_SIZE*R_BOARD_SIZE];   // Flat array for board
};
