/*
 *  connect4.h
 *
 *  Connect Four application window
 *
 *  Ken Reneris     12/04/2002
 *
 */


typedef struct {
    UINTN           Sig;
    BOOLEAN         Animate;                // Animate flips
    BOOLEAN         Sound;                  // Sound enabled/disabled
    BOOLEAN         Spare1[2];              // Align
    UINTN           Seed;
    UINTN           Depth[R_MAX_TYPE];
} C4Settings;

#define SETTINGS_OFFSET     128
#define SETTINGS_SIG        0xA12EB4D8


class CPoint {
public:
    inline CPoint &operator+=(CPoint &s1)   { x += s1.x;  y += s1.y;  return *this; }
    inline CPoint &operator-=(CPoint &s1)   { x -= s1.x;  y -= s1.y;  return *this; }
    inline BOOL operator==(CPoint &s1)      { return x == s1.x && y == s1.y;        }

public:
    INTN            x;
    INTN            y;
};


class C4App : public CWindow {
public:
    C4App();
    VOID LoadSettings();
    VOID SaveSettings();

    virtual S32 MsgHandler(MSG_TYPE type, CViewable *object, S32 data);
    virtual void Draw();

    // Callback from C4Brd
    VOID DrawProgressBar(INTN Count);

    // Menu options
    VOID NewGame();
    VOID UndoMove();

    VOID CheckNextMove();                   // Check to see who's turn it is


private:
    VOID MakeMove(UINTN Col);
    inline BOOLEAN IsHumanTurn()        { return m_Set.Depth[m_Brd.m_Color] == 0;   }
    inline BOOLEAN IsComputerTurn()     { return m_Set.Depth[m_Brd.m_Color] != 0;   }
    VOID ICheckNextMove();
    VOID FindWin(UINTN Color);
    VOID CheckRun(CPoint pt, CPoint d, UINTN Color);
    VOID Timer();

    VOID DrawTurn();
    VOID GameOver();
    VOID PlaySound(const piezo_tone *Snd);
    VOID PlayClick();

public:
    // Our settings
    C4Settings      m_Set;

private:
    ebo_name_t      m_SetName;              // Our settings file name
    CMenu           *m_FMenu;               // Our main "file menu"
    UINTN           m_FMenuUndo;            // Menu item that is "undo"
    CLabel          *m_StatLn1;             // Status line 1 label
    CLabel          *m_StatLn2;             // Status line 2 labal
    CProgressBar    *m_Bar;                 // Thinking progress bar
    CHAR            m_Ln1[80];
    CHAR            m_Ln2[80];

    // The current board
    C4Brd           m_Brd;

    // ICheckNextMove state
    BOOLEAN         m_Thinking;
    BOOLEAN         m_GameOver;             // Game is over
    INTN            m_WinFlash;             // Flash the winner
    INTN            m_WinRun;               // # of chips in Win
    CPoint          m_Win[60];              // Which cells to flash

    // Undo buffer
    UINTN           m_HistIndex;
    C4Brd           m_Hist[64];
};
