/*
 *  settings.cpp
 *
 *  Setting dialog
 *
 *  Ken Reneris     12/04/2002
 *
 */


#include "std.h"

const INTN R2Depth[] = { 0, 2, 4, 6 };
const INTN D2Row[]   = { 0, 0, 1, 1, 2, 2, 3 };

CRevSettings::CRevSettings()
    : CWindow(MAIN_WINDOW_ID, 10, 30, 180, 190, "Settings")
{
    INTN        x, y;
    CLabel    *LabelWhite = new CLabel(SETTING_WLABEL, "White Player");
    CLabel    *LabelBlack = new CLabel(SETTING_BLABEL, "Black Player");
    CMenu     *WPType     = new CMenu(SETTING_WPTYPE, 0, 0, 0, 0, 0 );
    CMenu     *BPType     = new CMenu(SETTING_BPTYPE, 0, 0, 0, 0, 0 );
    CButton   *ButOk      = new CButton(ID_OK, 0, 0, "OK");
    CButton   *ButCancel  = new CButton(ID_CANCEL, 0, 0, "Cancel");

    m_Sound   = new CCheckbox(SETTING_SOUND, 0, 0, "Sounds");
    m_Animate = new CCheckbox(SETTING_ANIMATE, 0, 0, "Animate drops");
    m_WPTypePop = SetPlayerPopup(SETTING_WPTYPE_TRIG, WPType, R_WHITE);
    m_BPTypePop = SetPlayerPopup(SETTING_BPTYPE_TRIG, BPType, R_BLACK);

    AddChild(LabelWhite, 5, 10);
    AddChild(LabelBlack, 5, 30);
    AddChild(m_Animate, 5, 80);
    AddChild(m_Sound, 5, 100);

    x = LabelWhite->GetX() + LabelWhite->GetWidth() + 5;
    AddChild(m_WPTypePop, x, 10);
    AddChild(m_BPTypePop, x, 30);

    // Add our OK & Cancel buttons
    y = GetHeight()-ButCancel->GetHeight()-25;
    x = GetWidth()-ButCancel->GetWidth()-20;
    AddChild(ButCancel, x, y);

    x = x - ButOk->GetWidth() - 20;
    AddChild(ButOk, x, y);

    //
    // Set our initial dialog control states
    //

    m_Sound->SetDownStatus(CApp->m_Set.Sound);
    m_Animate->SetDownStatus(CApp->m_Set.Animate);
}


CPopupTrigger *CRevSettings::SetPlayerPopup(UINTN id, CMenu *Menu, UINTN Color)
{
    UINTN           Type;

    Type = D2Row[CApp->m_Set.Depth[Color]];

    Menu->SetNumRows(4);
    Menu->SetRow(0, SETTING_HUMAN, "Human");
    Menu->SetRow(1, SETTING_EASY,  "Easy");
    Menu->SetRow(2, SETTING_MED,   "Medium");
    Menu->SetRow(3, SETTING_HARD,  "Take your time");
    Menu->SetCurrentRow(Type);
    return new CPopupTrigger(id, 0, 0, Menu, Type);
}



S32 CRevSettings::MsgHandler( MSG_TYPE type, CViewable *object, S32 data )
{
    switch (type) {
    case MSG_BUTTON_SELECT:
        switch(object->GetID()) {
        case ID_OK:
            CApp->m_Set.Sound = m_Sound->GetDownStatus();
            CApp->m_Set.Animate = m_Animate->GetDownStatus();
            CApp->m_Set.Depth[R_WHITE] = R2Depth[m_WPTypePop->GetCurrentRow()];
            CApp->m_Set.Depth[R_BLACK] = R2Depth[m_BPTypePop->GetCurrentRow()];
            CApp->SaveSettings();
            CApp->CheckNextMove();
            Close();
            break;

        case ID_CANCEL:
            Close();
            break;
        }
        break;

    // Msg not handled
    default:
        return 0;
    }

    return 1;
}
